import { Context } from "../context/context.js";
import { useCookie, useHeader } from "../context/http.js";
import { createSigner } from "fast-jwt";
import { KEY, verifier } from "./jwt.js";
const SessionMemo = /* @__PURE__ */ Context.memo(() => {
    let token = "";
    const header = useHeader("authorization");
    if (header)
        token = header.substring(7);
    const cookie = useCookie("auth-token");
    if (cookie)
        token = cookie;
    if (token) {
        const jwt = verifier(token);
        return jwt;
    }
    return {
        type: "public",
        properties: {}
    };
});
// This is a crazy TS hack to prevent the types from being evaluated too soon
export function useSession() {
    const ctx = SessionMemo();
    return ctx;
}
function create(input) {
    const signer = createSigner(Object.assign(input.options || {}, { key: KEY }));
    const token = signer({
        type: input.type,
        properties: input.properties
    });
    return token;
}
export function cookie(input) {
    const token = create(input);
    const expires = new Date(Date.now() + (input.options?.expiresIn || 1000 * 60 * 60 * 24 * 7));
    return {
        statusCode: 302,
        headers: {
            location: input.redirect
        },
        cookies: [
            `auth-token=${token}; HttpOnly; SameSite=None; Secure; Path=/; Expires=${expires}`
        ]
    };
}
export function parameter(input) {
    const token = create(input);
    return {
        statusCode: 302,
        headers: {
            location: input.redirect + "?token=" + token
        }
    };
}
export const Session = {
    create,
    cookie,
    parameter
};
