"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.config = exports.Config = void 0;
const ssm_js_1 = __importDefault(require("aws-sdk/clients/ssm.js"));
const ssm = new ssm_js_1.default();
async function wildcard(prefix, token) {
    const result = await ssm
        .getParametersByPath({
        WithDecryption: true,
        Recursive: true,
        Path: prefix,
        NextToken: token,
    })
        .promise();
    const parameters = result.Parameters || [];
    return [
        ...parameters,
        ...(result.NextToken ? await wildcard(prefix, result.NextToken) : []),
    ];
}
async function load() {
    const splits = process.env.SSM_VALUES.split(",");
    const work = [];
    while (splits.length) {
        const chunk = splits.splice(0, 10);
        work.push((async () => {
            const result = await ssm
                .getParameters({
                Names: chunk.map((v) => `${process.env.SSM_PREFIX}${v}`),
                WithDecryption: true,
            })
                .promise();
            const params = result.Parameters || [];
            if (result.InvalidParameters?.length) {
                if (!process.env.SSM_FALLBACK)
                    throw new Error("Missing parameters and no fallback specified: " +
                        result.InvalidParameters.join(", "));
                const fallback = await ssm
                    .getParameters({
                    Names: (result.InvalidParameters || []).map((item) => item.replace(process.env.SSM_PREFIX, process.env.SSM_FALLBACK)),
                    WithDecryption: true,
                })
                    .promise();
                if (fallback.InvalidParameters?.length)
                    throw new Error("Missing parameters after searching fallback: " +
                        fallback.InvalidParameters.join(", "));
                params.push(...(fallback.Parameters || []));
            }
            return params;
        })());
    }
    const all = await Promise.all(work);
    return all.flat();
}
if (!process.env.SSM_PREFIX)
    throw new Error("SSM_PREFIX is not set");
exports.Config = {};
const params = process.env.SSM_VALUES === "*"
    ? [
        ...(await wildcard(process.env.SSM_PREFIX)),
        ...(process.env.SSM_FALLBACK
            ? await wildcard(process.env.SSM_FALLBACK)
            : []),
    ]
    : await load();
for (const item of params || []) {
    const last = item.Name.split("/").pop();
    exports.Config[last] = process.env[last] || item.Value;
}
function config(key) {
    const value = exports.Config[key];
    if (!value)
        throw new Error(`Missing parameter "${key}"`);
    return value;
}
exports.config = config;
